% This script plots eigenvalues and eigenfunctions created using the script
% 'equamodes.m'. Note that this script only plots eigenfunctions of the
% complete equation set or the geostrophic modes, not modes under the WTG
% approximation. Only the largest growth rates for each mode number and
% zonal wavenumber are plotted.
% August, 2019
%
ep=0.8;  % Precipitation efficiency
%--------------------------------------------------------------------------
load equatemp  % Loads eigenvalues and other quantities created by equamode
qn=max(size(N));
qk2=max(size(k2));
close all
clear Ad
%
% Start off by replotting eigenvalue figures
%
uf=openfig('figtemp');
set(uf,'Position',[50 275 600 450])
%
uf2=openfig('figtemplow');
set(uf2,'Position',[700 275 600 450])
%
% Ask for n and k values for eigenfunction plots
%
jm=1;
while jm > 0
    jk=1;
    while jk > 0
        prompt = {'Enter merdional mode number (-1,0,1,2):','Enter zonal wavenumber: (-10 to 10)', ...
            'Plot full (f) or geostropic (g) eigenfunctions'};
        dlgtitle = 'Make eigenfunction plot';
        dims = [1 70];
        Ad = inputdlg(prompt,dlgtitle,dims);
        if isempty(Ad) || isempty(Ad{2})
            jm=0;
            break
        end 
        try
           close(uf)
        catch
        end
        nm=str2num(Ad{1}); %#ok<ST2NM>
        km=str2num(Ad{2}); %#ok<ST2NM>
        plotmode=Ad{3};
        [~,jm]=min(abs(nm-N)); % Calculate index for n value
        [~,im]=min(abs(km-k2)); % Calculate index for k2 value
        [~,imk]=min(abs(abs(km)-k)); % Calculate index for k value
        %
        if strcmp(plotmode,'f')
            sig=sigsort(jm,im,1); % Note that only largest growth rate models plotted
        else
            sig=sigsortg(jm,im,1);
        end    
        %
        if isnan(sig)
            msgbox('No modes for this combination of mode number and wavenumber')
            pause(3)
            break
        end    
        km=abs(km);
        %
        % Recalculate meridional decay factor using chosen root for sig. Then
        % figure out which root of meridional decay factor to use. 
        %
        a1=f1.*sig.^2+f2(imk).*sig+f3(imk);
        a2=f4.*sig+f5(imk);
        a3=f6.*sig+f7(imk);
        det=sqrt(a2^2+4.*a1*sig*a3);
        b1=(-a2+det)/(4*sig*a3);
        b2=(-a2-det)/(4*sig*a3);
        if nm > -1 && strcmp(plotmode,'f')
            fac=(sig/delta)*(a1*sig+ai*km*a2+a3*km^2);
        else 
            fac=0;
        end    
        res1=ai.*km*a3-0.5*a2-(nm+0.5)*det-fac;
        res2=ai.*km*a3-0.5*a2+(nm+0.5)*det-fac;
        if nm > -1 && min(abs(res1),abs(res2)) > 1e-4  % (This should never be invoked!)
            msgbox(['Warning: Residual not small, Residual = ',num2str(min(abs(res1),abs(res2)))])
        end    
        b=b1;
        if abs(res2) < abs(res1) && b2 > 0
            b=b2;
        end 
        %
        %   Calculate and plot eigenfunctions
        %
        y=-4:0.05:4;   % Note choice of range of y to plot here
        x=0:0.05:2*pi; % Full wavelenth in x
        qy=max(size(y));
        qx=max(size(x));
        if km ~= 0
            X=exp(ai.*x); % We plot one full wavelength in x regardless of k value
        else
            X=zeros(1,qx)+1;
        end    
        %
        F=zeros(qx,qy);
        %
        % Now caluclate eigenfunctions
        %
        if nm == -1  
            Y=exp(0.5*y.^2.*ai*km/sig);
            for i=1:qx
                for j=1:qy
                    F(i,j)=X(i).*Y(j);
                end
            end
            %
            V=zeros(qx,qy);
            U=real(F);
            W=real(-ai.*km.*F);
            S=real(sig.*F./(ai.*km));
            Sm=real(((chi+sig).*(sig.*F./(ai.*km))+ai.*km.*F+alpha.*F)/(1+C));
            Vor=real(-ai.*km.*F./sig).*repmat(y,[qx,1]);
            M=W+((1-ep)/ep)*(Sm-chi*S-alpha*U);
        else
            nm1=nm+1;
            Y=exp(-b.*y.^2);
            %
            % First calculate Hermite polynomials and their first derivatives
            %
            H=zeros(4,qy);
            HD=zeros(4,qy);
            H(1,1:qy)=1;
            H(2,1:qy)=y;
            if strcmp(plotmode,'f')
                H(3,1:qy)=0.5.*y.^2+sig*a3/(0.5*a1/b-ai*abs(km)*a3);
                c2=2.*sig*a3/(6*b*sig*a3+2.*a2-ai*km*a3);
            else
                H(3,1:qy)=0.5.*y.^2+sig*a3/(2.*b*sig*a3-ai*km*a3+a2+(sig/delta)*( ...
                    a1*sig+ai*km*a2+km^2*a3));
                c2=2.*sig*a3/(6*b*sig*a3+2.*a2-ai*km*a3+(sig/delta)*( ...
                    a1*sig+ai*km*a2+km^2*a3));
            end    
            H(4,1:qy)=(y.^3)/3+c2.*y;
            HD(1,1:qy)=0;
            HD(2,1:qy)=1;
            HD(3,1:qy)=y;
            HD(4,1:qy)=y.^2+c2;
            %
            % Now calculate eigenfunctions
            %
            sraw=zeros(qx,qy);
            uraw=zeros(qx,qy);
            vorraw=zeros(qx,qy);
            wraw=zeros(qx,qy);
            for i=1:qx
                for j=1:qy
                    F(i,j)=X(i)*Y(j)*H(nm1,j);
                    if nm == 0
                        sraw(i,j)=y(j)*X(i)*Y(j)*sig/delta;
                        uraw(i,j)=(ai*km*sraw(i,j)+y(j)*F(i,j))/sig;
                        wraw(i,j)=2*b*y(j)*F(i,j)-ai*km*uraw(i,j);
                        vorraw(i,j)=-X(i)*Y(j)*(1-4*b*y(j)^2+(4/3)*b^2*y(j)^4)/sig;
                        vorraw(i,j)=vorraw(i,j)+ai*km*F(i,j)/delta;
                    else    
                        sraw(i,j)=X(i)*Y(j)*(a3*(HD(nm1,j)-2*b*y(j)*H(nm1,j))+ ...
                            y(j)*H(nm1,j)*(ai*km*a3-a2)/sig)/(a1+ai*km* ...
                            (a2-ai*km*a3)/sig);
                        uraw(i,j)=(ai*km.*sraw(i,j)+y(j)*F(i,j))/sig;
                        wraw(i,j)=-(a2*uraw(i,j)+a1*sraw(i,j))/a3;
                        vorraw(i,j)=-(ai*km*y(j)*uraw(i,j)+F(i,j)+y(j)*X(i)*Y(j).* ...
                            (HD(nm1,j)-2.*b.*y(j).*H(nm1,j)))/sig;
                        vorraw(i,j)=vorraw(i,j)+ai*km*F(i,j)/delta;
                    end
                end
            end   
            S=real(sraw);
            U=real(uraw);
            V=real(F);
            Vor=real(vorraw);
            W=real(wraw);
            Sm=real((chi+sig)*sraw+alpha*uraw-(a2*uraw+a1.*sraw)/a3)/(1+C);
            M=W+((1-ep)/ep).*(Sm-chi*S-alpha*U);
        end    
        %
        %  Now make plots
        %
        indx=1;
        items={'Full eigenvalue plot','Slow frequencies only', ...
            'Geostrophic approximation','Geostrophic and WTG approximation', ...
            'Summary eigenfunction plot', ...
            'Merdional wind','Zonal wind','Vorticity','Vertical wind', ...
            'Convective Mass Flux', ...
            'Mid-tropospheric moist entropy','Saturation entropy'}; 
        while isempty(indx) == 0 
            [indx,tf] = listdlg('PromptString','Select a plot:',...
                                       'SelectionMode','single',...
                                       'CancelString','Stop', ...
                                       'ListString',items);
            if tf == 0 || isempty(Ad)
                break
            end
            close all
            if indx == 1
                uf=openfig('figtemp');
                set(uf,'Position',[50 275 600 450])
            elseif indx == 2
                uf=openfig('figtemplow');
                set(uf,'Position',[50 275 600 450])
            elseif indx == 3
                uf=openfig('figtemp');
                set(uf,'Position',[50 275 600 450])
                hold on
                for m=1:qn
                    for i=1:qk2  % Plot frequencies with marker sizes proportional to growth rates
                        msize=round(msizefac*real(sigsortg(m,i,1)));
                        msize=0.3*msize;
                        msize=max(msize,10);
                        if N(m) ~= 0 && abs(imag(sigsortg(m,i,1))) >= 1e-4
                            plot(k2(i),(abs(imag(sigsortg(m,i,1)))),'o','color',col(m,:),'markersize',msize)
                        end   
                    end            
                end
                hold off
                title('Full and geostrophic eigenvalue spectra')
            elseif indx == 4
                uf=openfig('figtemp');
                set(uf,'Position',[50 275 600 450])
                hold on
                for m=1:qn
                    for i=1:qk2  % Plot frequencies with marker sizes proportional to growth rates
                        msize=round(msizefac*real(sigsortwtg(m,i)));
                        msize=0.3*msize;
                        msize=max(msize,10);
                        if N(m) ~= 0 && abs(imag(sigsortwtg(m,i,1))) >= 1e-4
                            plot(k2(i),(abs(imag(sigsortwtg(m,i)))),'o','color',col(m,:),'markersize',msize)
                        end   
                    end            
                end
                hold off
                title('Full and WTG eigenvalue spectra')
            elseif indx == 5
                figure('Position',[50 275 600 450])
                pcolor(x,y,W')
                shading interp
                hold on
                quiver(x(1:5:end),y(1:5:end),U(1:5:end,1:5:end)',V(1:5:end,1:5:end)',1.5,'color','b')
                colorbar
                colormap('jet')
                contour(x,y,max(S',0),8,'k');
                contour(x,y,-min(S',0),8,'--k');
                hold off
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Winds (arrows), w (colors) and s* (contours)')        
            elseif indx == 6
                if nm == -1  % Do not plot V for n=-1 mode for which it is zero
                    msgbox('No V field for n = -1 mode')
                else
                    figure('Position',[50 275 600 450])
                    contourf(x,y,V',10)
                    set(gca,'fontsize',12,'fontweight','bold')
                    xlabel('x')
                    ylabel('y','rotation',0)
                    title('Meridional velocity')
                    colormap('gray')
                    colorbar
                end
            elseif indx == 7
                figure('Position',[50 275 600 450])
                contourf(x,y,U',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Zonal velocity')
                colormap('gray')
                colorbar
            elseif indx == 8
                figure('Position',[50 275 600 450])
                contourf(x,y,Vor',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Vorticity')
                colormap('gray')
                colorbar
            elseif indx == 9    
                figure('Position',[50 275 600 450])
                contourf(x,y,W',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Vertical velocity')
                colormap('gray')
                colorbar
            elseif indx == 10    
                figure('Position',[50 275 600 450])
                contourf(x,y,M',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Convective mass flux')
                colormap('gray')
                colorbar
            elseif indx == 11
                figure('Position',[50 275 600 450])
                contourf(x,y,Sm',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Mid-troposphere moist entropy')
                colormap('gray')
                colorbar
            elseif indx == 12
                figure('Position',[50 275 600 450])
                contourf(x,y,S',10)
                set(gca,'fontsize',12,'fontweight','bold')
                xlabel('x')
                ylabel('y','rotation',0)
                title('Saturation entropy')
                colormap('gray')
                colorbar
            end
        end    
    end
end